# Copyright (C) 2005  Network Applied Communication Laboratory Co., Ltd.
#
# This file is part of Rast.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA

testdir = File.dirname(__FILE__)
$:.unshift(testdir)

at_exit do
  GC.start # to avoid SEGV on finalizers of bdb.
end

require "find"
require "optparse"

TEST_SCRIPT_REGEXP = /-test\.rb\z/

target_directory = testdir
pattern = ""

verbose = false

OptionParser.new do |opts|
  opts.banner = "Usage: #{File.basename($0)} [options]"

  opts.separator("")
  opts.separator("Specific options:")

  opts.on("--target-directory TARGET", "set target directory.") do |arg|
    target_directory = arg
  end

  opts.on("-I PATH", "add PATH to load-path ($:).") do |arg|
    $:.push(arg)
  end

  opts.on("--pattern PATTERN", "specify pattern for test classes") do |arg|
    pattern = arg
  end

  opts.on("--verbose", "verbose output") do
    verbose = true
  end

  opts.parse!(ARGV)
end

pattern_re = Regexp.new(pattern)

puts if verbose
Find.find(target_directory) do |name|
  if File.file?(name)
    if TEST_SCRIPT_REGEXP =~ name && pattern_re =~ name
      if verbose
        printf("require '%s'\n", name)
      end
      require name
    end
  end
end
puts if verbose
